import {
  globalExpect,
  init_dist,
  test
} from "../chunk-KW47YOIJ.mjs";
import {
  init_internal
} from "../chunk-7ANA32NV.mjs";
import {
  systemLogger
} from "../chunk-ATZ7N7EG.mjs";
import {
  __commonJS
} from "../chunk-C6P2IO65.mjs";

// src/lib/system_logger.test.ts
import process from "node:process";
var require_system_logger_test = __commonJS({
  "src/lib/system_logger.test.ts"() {
    init_dist();
    init_internal();
    test("Log Level", () => {
      const originalDebug = console.debug;
      const debugLogs = [];
      console.debug = (...message) => debugLogs.push(message);
      systemLogger.debug("hello!");
      globalExpect(debugLogs.length).toBe(0);
      systemLogger.withLogLevel(1 /* Debug */).debug("hello!");
      globalExpect(debugLogs.length).toBe(1);
      systemLogger.withLogLevel(2 /* Log */).debug("hello!");
      globalExpect(debugLogs.length).toBe(1);
      console.debug = originalDebug;
    });
    test("Fields", () => {
      const originalLog = console.log;
      const logs = [];
      console.log = (...message) => logs.push(message);
      systemLogger.withError(new Error("boom")).withFields({ foo: "bar" }).log("hello!");
      globalExpect(logs.length).toBe(1);
      globalExpect(logs[0][0]).toBe("__nfSystemLog");
      const log = JSON.parse(logs[0][1]);
      globalExpect(log.msg).toBe("hello!");
      globalExpect(log.fields.foo).toBe("bar");
      globalExpect(log.fields.error).toBe("boom");
      globalExpect(log.fields.error_stack.split("\n").length > 2).toBe(true);
      console.log = originalLog;
    });
    test("Local Dev", () => {
      const originalLog = console.log;
      const logs = [];
      console.log = (...message) => logs.push(message);
      systemLogger.log("hello!");
      globalExpect(logs.length).toBe(1);
      process.env.NETLIFY_DEV = "true";
      systemLogger.log("hello!");
      globalExpect(logs.length).toBe(1);
      process.env.NETLIFY_ENABLE_SYSTEM_LOGGING = "true";
      systemLogger.log("hello!");
      globalExpect(logs.length).toBe(2);
      delete process.env.NETLIFY_DEV;
      delete process.env.NETLIFY_ENABLE_SYSTEM_LOGGING;
      console.log = originalLog;
    });
  }
});
export default require_system_logger_test();
